package oop2visuals;

import processing.core.PApplet;
import oop2visuals.Drawable;

public class House implements Drawable{

	Basement base;
	Storey[] storeys;
	Roof     roof;
	
	public House (Basement b, Storey[] ss, Roof r) {
		base = b;
		storeys = ss;
		roof = r;
	}

	@Override
	public void draw(PApplet canvas, float posX, float posY) {
		canvas.stroke(0, 0, 0);
		float h = posY;
		base.draw(canvas, posX, h);
		h -= base.height*scale;
		for (int i = 0; i < storeys.length; i++){
			storeys[i].draw(canvas, posX, h);
			h -= storeys[i].height*scale;
		}
		roof.draw(canvas, posX, h);
	}
	
	
}

// a base for our house
class Basement implements Drawable {
	
	float width = 20;
	float height = 1;
	int colourR = 10, colourG = 10, colourB = 10;
	
	@Override
	public void draw(PApplet canvas, float posX, float posY) {
		canvas.fill(this.colourR, this.colourG, this.colourB);
		canvas.rect( posX
				   , posY - scale*this.height
				   , scale*this.width
				   , scale*this.height);
	}
	
	
}

// a single floor
class Storey implements Drawable{
	float width = 20;
	float height = 3;
	int colourR = 180, colourG = 80, colourB = 10;
	
	@Override
	public void draw(PApplet canvas, float posX, float posY) {
		// setup color
		canvas.fill(this.colourR, this.colourG, this.colourB);
		// draw floor
		canvas.rect( posX
				   , posY - scale*this.height
				   , scale*this.width
				   , scale*this.height);
		// set window color
		smartTurnLightsOn(canvas);
		// draw windows
		for (int i = 0; i < numberOfWindows(); i++) {
			canvas.rect( posX + scale*((2+windowSeperator())*i + windowSeperator()*0.5f)
					   , posY - scale*this.height*0.8f
					   , scale*2
					   , scale*this.height*0.5f);
		}
	}
	
	int numberOfWindows() { 
		return Math.round((float)Math.floor(this.width / 3));
	}
	
	float windowSeperator() {
		return (this.width - numberOfWindows()*2) / numberOfWindows();
	}
	
	private Human owner = null;
	public void registerOwner(Human man) {
		this.owner = man;
	}
	
	void smartTurnLightsOn(PApplet canvas) {
		if(this.owner != null && this.owner.state == Human.State.AT_HOME) {
			canvas.fill(255, 255, 211);
		} else {
			canvas.fill(188, 188, 211);
		}
	}
}


// ground floor
class GroundFloor extends Storey {

	@Override
	public void draw(PApplet canvas, float posX, float posY) {
		// first, let's draw all windows as in super class
		super.draw(canvas, posX, posY);
		// next, draw a door
		canvas.fill(100, 0, 0);
		int i = numberOfWindows() / 2;
		canvas.rect( posX + scale*((2+windowSeperator())*i + windowSeperator()*0.5f)
				   , posY - scale*this.height*0.8f
				   , scale*2
				   , scale*this.height*0.8f);
	}
	
}

// roof - abstract, because there are different types of roofs
abstract class Roof implements Drawable{
	float width = 20;
	float height = 0.5f;
	int colourR = 0, colourG = 80, colourB = 80;
	
}


class FlatRoof extends Roof {
	
	@Override
	public void draw(PApplet canvas, float posX, float posY) {
		canvas.fill(this.colourR, this.colourG, this.colourB);
		canvas.rect( posX - scale*0.5f
				   , posY - scale*this.height
				   , scale*this.width + scale*1.0f
				   , scale*this.height);
	}
	
}

class SlopedRoof extends Roof {

	float height = 4f;
	
	@Override
	public void draw(PApplet canvas, float posX, float posY) {
		canvas.fill(this.colourR, this.colourG, this.colourB);
		canvas.triangle( posX - scale*1, posY
				       , posX + scale*(this.width + 1), posY
				       , posX + scale*this.width/2, posY - scale*this.height
				       );
	}
	
}